#ifndef FACTORIES_H
#define FACTORIES_H

class Formula;
class WorkSpace;

class Factory {
  WorkSpace* ws;
 public:
  Factory() : ws(0) { }
  Factory(WorkSpace& workSpace) : ws(&workSpace) { }
  void setWorkSpace(WorkSpace& workSpace) { ws = &workSpace; }
  bool hasWorkSpace() { return ws!=0; }
  WorkSpace& workSpace();
  const WorkSpace& workSpace() const;
  virtual Formula* makeConstant(bool Polarity) = 0;
  virtual Formula* makeALiteral(int value) = 0;
  virtual Formula* makeJunction(Formula* lhs, Formula* rhs, bool Polarity) = 0;
  // quick shortcuts
  Formula* makeTop() { return makeConstant(true); }
  Formula* makeBot() { return makeConstant(false); }
  Formula* makeLiteral(int value, bool polarity)
    { if (!polarity) value = -value; return makeALiteral(value); }
  Formula* makeConjunction(Formula* lhs, Formula* rhs) 
    { return makeJunction(lhs, rhs, true); }
  Formula* makeDisjunction(Formula* lhs, Formula* rhs)
    { return makeJunction(lhs, rhs, false); }
  // common algorithms
  Formula* copy(const Formula& formula, bool polarity = true);
  Formula* filter(Formula* formula, bool polarity = true);
};

class FormulaMaker {
  Factory* factory_;
 public:
  FormulaMaker(Factory* factory = 0) : factory_(factory) { };
  virtual ~FormulaMaker();
  Factory& factory();
  void setFactory(Factory* factory);
  bool factoryReady() { return factory_!=0; }
};

class BasicFactory : public Factory {
 public:
  BasicFactory() : Factory() { }
  BasicFactory(WorkSpace& workSpace) : Factory(workSpace) { }
  Formula* makeConstant(bool polarity);
  Formula* makeALiteral(int value);
  Formula* makeJunction(Formula* lhs, Formula* rhs, bool polarity);
};

class SimpleFactory : public BasicFactory {
 public:
  SimpleFactory() : BasicFactory() { }
  SimpleFactory(WorkSpace& workSpace) : BasicFactory(workSpace) { }
  Formula* makeJunction(Formula* lhs, Formula* rhs, bool polarity);
};

class CnfFactory : public Factory {
 public:
  CnfFactory() : Factory() { }
  CnfFactory(WorkSpace& workSpace) : Factory(workSpace) { }
  Formula* makeConstant(bool polarity);
  Formula* makeALiteral(int value);
  Formula* makeJunction(Formula* lhs, Formula* rhs, bool polarity);
};

class OptimizedClausalFactory : public CnfFactory {
 public:
  OptimizedClausalFactory() : CnfFactory() { }
  OptimizedClausalFactory(WorkSpace& workSpace) : CnfFactory(workSpace) { }
  Formula* nameFormula(Formula* rhs);
  Formula* makeJunction(Formula* lhs, Formula* rhs, bool polarity);
};

#endif // FACTORIES_H
