#include "deffmas.h"
#include <cassert>
#include <stdexcept>

using namespace std;

// class Formula
Formula::Formula() { }

Formula::~Formula() { }

void Formula::getChilds(Formula& parent) {
  if (&parent == this)
    throw logic_error("Can't get my own childs into myself!");
  while (parent.count()) { push(parent.pop()); }
}

void Formula::getChildCopies(const Formula& parent) {
  for (int i=0; i<parent.count(); i++)
    push(parent[i].clone());
}

// class Literal
Literal::Literal(int value) : value_(value) { assert(value); }

FormulaId Literal::classId() const { return idLiteral; }

void Literal::clear() { }

int Literal::count() const { return 0; }

int Literal::value() const { return value_; }

bool Literal::polarity() const { return (value_>0); }

void Literal::push(Formula* child) { 
  throw logic_error("Can't push childs into a literal!");
}

Formula* Literal::pop() {
  throw logic_error("Can't pop childs from a literal!");
}

Formula* Literal::clone() const { return new Literal(value_); }

const Formula& Literal::operator[](int index) const { 
  throw logic_error("Index out of range");
}

Formula& Literal::operator[](int index) { 
  throw logic_error("Index out of range");
}

// class Junction
Junction::Junction(bool polarity) : polarity_(polarity) { }

Junction::Junction(Formula* child, bool polarity) : polarity_(polarity) {
  push(child);
}

Junction::~Junction() { clear(); }

FormulaId Junction::classId() const { return idJunction; }

void Junction::clear() {
  for (int i=0; i<count(); i++)
    delete childs[i];
  
  childs.clear();
}

int Junction::count() const { return childs.size(); };

int Junction::value() const { return 0; }

bool Junction::polarity() const { return polarity_; }

void Junction::push(Formula* child) { childs.push_back(child); }

Formula* Junction::pop() {
  Formula* result = childs[count()-1];
  childs.pop_back();
  return result;
}

const Formula& Junction::operator[](int index) const { return *childs[index]; }

Formula& Junction::operator[](int index) { return *childs[index]; }

Formula* Junction::clone() const {
  Formula* result = new Junction(polarity());

  for (int i=0; i<count(); i++)
    result->push(operator[](i).clone());
  
  return result;
}


