#ifndef FORMULAS_H
#define FORMULAS_H

enum FormulaId { idLiteral, idJunction };

class Formula {
  Formula(const Formula& rhs);             // disable copy constructor
  Formula& operator=(const Formula& rhs);  // disable assignment operator
 public:
  Formula();
  virtual ~Formula();
  virtual FormulaId classId() const = 0;
  virtual void clear() = 0;
  virtual int count() const = 0;
  virtual int value() const = 0;
  virtual bool polarity() const = 0;
  virtual void push(Formula* child) = 0;
  virtual Formula* pop() = 0;
  virtual const Formula& operator[](int index) const = 0;
  virtual Formula& operator[](int index) = 0;
  virtual Formula* clone() const = 0;
  void getChilds(Formula& parent);
  void getChildCopies(const Formula& parent);
};

inline bool IsJunction(const Formula& f, bool polarity) {
  return (f.classId()==idJunction) and (f.polarity()==polarity);
}

inline bool IsConstant(const Formula& f, bool polarity) {
  return IsJunction(f, polarity) and (f.count()==0);
}

inline bool IsLiteral(const Formula& f) {
  return f.classId()==idLiteral;
}

inline bool IsConjunction(const Formula& f) {
  return IsJunction(f, true);
}

inline bool IsDisjunction(const Formula& f) {
  return IsJunction(f, false);
}

inline bool IsTop(const Formula& f) {
  return IsConstant(f, true);
}

inline bool IsBot(const Formula& f) {
  return IsConstant(f, false);
}

#endif // FORMULAS_H
