#include <set>
#include "generators.h"
#include "workspace.h"
#include "utils.h"

using namespace std;

// class Generator
Formula* Generator::makeFormulas(int count) {
  Formula* result = factory().makeTop();
  for (int i=0; i<count; i++)
    result = factory().makeConjunction(result, makeFormula());
  return result;
}

// class SillyRandom
Formula* SillyRandom::make(int depth) {
  if (!depth) {
    if (randInt(6)==0)
      return factory().makeConstant(randInt(2));
    else
      return factory()
	.makeLiteral(randInt(ws.signature().userVariables()) + 1,
		     randInt(2));
  } else {
    depth--;
    return factory().makeJunction(make(depth), make(depth), randInt(2));
  }
}

Formula* SillyRandom::makeFormula() { return make(maxdepth); }

// class RegularRandom

Formula* RegularRandom::make(int level) {
  if (level==list.size()) {
    return factory().makeLiteral(randInt(ws.signature().userVariables()) + 1,
				 randInt(2));
  } else {
    Formula* result = factory().makeConstant(level%2);
    for (int i=0; i<list[level]; i++)
      result = factory().makeJunction(result, make(level+1), level%2);
    return result;
  }
}

Formula* RegularRandom::makeFormula() { return make(0); }

// class BalancedRandom

int BalancedRandom::RandomVar() { 
	return randInt(ws.signature().userVariables()) + 1; 
}

Formula* BalancedRandom::make(int level) {
	Formula* result = factory().makeConstant(level%2);
	if (level==list.size()-1) {
		std::set<int> lits;
		while (lits.size() < list[level]) lits.insert(RandomVar());
		for (std::set<int>::iterator i = lits.begin(); i!= lits.end(); i++)
			result = factory().makeJunction(
				result, 
        factory().makeLiteral(*i, randInt(2)), level%2
      );
	}	else {
    for (int i=0; i<list[level]; i++)
      result = factory().makeJunction(result, make(level+1), level%2);
  }
	return result;
}

Formula* BalancedRandom::makeFormula() { return make(0); }


