#ifndef GENERATORS_H
#define GENERATORS_H

#include "factories.h"
#include <vector>

class Formula;
class WorkSpace;
typedef unsigned int uint;

class Generator : public FormulaMaker {
 public:
  Generator(Factory* factory = 0) : FormulaMaker(factory) { };
  virtual Formula* makeFormula() = 0;
  Formula* makeFormulas(int count);
};

class SillyRandom : public Generator {
  int maxdepth;
  WorkSpace& ws;
 protected:
  Formula* make(int depth);
 public:
  SillyRandom(int depth, WorkSpace& workSpace, Factory* factory = 0) : 
    Generator(factory), maxdepth(depth), ws(workSpace) { }
  Formula* makeFormula();
};

typedef std::vector<uint> RegShape;

class RegularRandom : public Generator {
  RegShape list;
  WorkSpace& ws;
 protected:
  Formula* make(int level);
 public:
  RegularRandom(const RegShape& shape, WorkSpace& workSpace, 
		Factory* factory = 0) :
    Generator(factory), list(shape), ws(workSpace) { }
	Formula* makeFormula();
};

class BalancedRandom : public Generator {
	RegShape list;
	WorkSpace& ws;
 protected:
	Formula* make(int level);
	int RandomVar();
 public:
	BalancedRandom(const RegShape& shape, WorkSpace& workSpace, Factory* factory = 0) :
		Generator(factory), list(shape), ws(workSpace) { }
	Formula* makeFormula();
};

#endif // GENERATORS_H
