#ifndef OPTIONS_H
#define OPTIONS_H

#include <string>
#include <vector>
#include <map>

class OptionsList {
public:
  static uint indent;
  static uint width;
  ~OptionsList() { clear(); }
  void clear();
  void AddOption(
    const std::string& keys,
    const std::string& help,
    const std::string& allowed_types = "",
    const std::string& default_value = "",
    bool optional = false
  );
  const std::string& operator[](const std::string& index) const;
  bool flag(const std::string& index) const;
  void DumpOptionsHelp(std::ostream& os) const;
  bool ProcessOption(const std::string& param);
  int ProcessOptions(int argc, char* argv[], int i = 1);
private:
  class Option;
  std::vector<Option*> list;
  std::map<std::string, Option*> params;
  class Option {
  public:
    std::string help;
    std::string value;
    std::string prototype;
    std::string allowed_types;
    bool flag;
    bool optional;
    void AddKey(const std::string& key);
    void DumpHelp(std::ostream& os) const;
  };
};

inline std::ostream& operator<<(std::ostream& os, const OptionsList& opts) {
  opts.DumpOptionsHelp(os);
  return os;
}

#endif // OPTIONS_H
