#include "workspace.h"
#include "deffmas.h"
#include "factories.h"
#include <stdexcept>


// class FixedSignature
int FixedSignature::variables() const { return fresh; }
int FixedSignature::userVariables() const { return user; }
int FixedSignature::systemVariables() const { return fresh - user; }
int FixedSignature::getFreshVariable() { return ++fresh; }

// class WorkSpace
WorkSpace::WorkSpace(const std::string& description, Signature* signature) :
  program_(0), signature_(signature), description_(description) { 
  if (!signature_) signature_ = new FixedSignature(); 
}

WorkSpace::~WorkSpace() {
  delete program_;
  delete signature_;
}

Formula& WorkSpace::program() {
  if (!program_) program_ = new Junction(true);
  return *program_;
}

const Formula& WorkSpace::program() const {
  if (!program_)
    throw std::logic_error("This WorkSpace does not have a Program!");
  return *program_;
}

void WorkSpace::setProgram(Formula* formula) {
  delete program_;
  program_ = formula;
}

Formula* WorkSpace::takeProgram() {
  Formula* result = program_;
  program_ = 0;
  return result;
}

void WorkSpace::addFormula(Formula* formula, Factory& factory) {
  if (!program_) 
    setProgram(formula);
  else {
    setProgram(factory.makeConjunction(takeProgram(), formula));
  }
}

void WorkSpace::addFormula(Formula* formula) {
  Factory* factory = new SimpleFactory(*this);
  addFormula(formula, *factory);
  delete factory;
}
