#ifndef WORKSPACE_H
#define WORKSPACE_H

#include <string>
class Formula;
class Factory;

class Signature {
 public:
  virtual int variables() const = 0;
  virtual int userVariables() const = 0;
  virtual int systemVariables() const = 0;
  virtual int getFreshVariable() = 0;
};

class FixedSignature : public Signature {
  int user, fresh;
 public:
  FixedSignature(int variables = 100) : user(variables), fresh(variables) { }
  int variables() const;
  int userVariables() const;
  int systemVariables() const;
  int getFreshVariable();
};

class WorkSpace {
  Formula* program_;
  Signature* signature_;
  std::string description_;
 public:
  WorkSpace(const std::string& description = "", Signature* signature = 0);
  ~WorkSpace();
  Formula& program();
  const Formula& program() const;
  void setProgram(Formula* formula);
  Formula* takeProgram();
  void addFormula(Formula* formula, Factory& factory);
  void addFormula(Formula* formula);
  Signature& signature() { return *signature_; }
  const Signature& signature() const { return *signature_; }
  std::string& description() { return description_; }
  const std::string& description() const { return description_; }
};

#endif // WORKSPACE_H
