#ifndef WRITERS_H
#define WRITERS_H

#include <iosfwd>
class Formula;
class WorkSpace;

std::ostream& operator<<(std::ostream& os, const Formula& formula);
std::ostream& operator<<(std::ostream& os, const WorkSpace& workspace);

class Writer {
 public:
  virtual void formulaToStream(const Formula& formula, std::ostream& os) = 0;
  virtual void workspaceToStream(const WorkSpace& workspace, std::ostream& os)
    = 0;
};

class SystemWriter {
  Writer* writer_;
 public:
  SystemWriter() : writer_(0) { }
  ~SystemWriter() { delete writer_; }
  Writer& writer();
  void setWriter(Writer* writer);
};

extern SystemWriter systemWriter;

class BasicWriter : public Writer {
 public:
  void formulaToStream(const Formula& formula, std::ostream& os);
  void workspaceToStream(const WorkSpace& workspace, std::ostream& os);
};

class CnfWriter : public Writer {
 public:
  void formulaToStream(const Formula& formula, std::ostream& os);
  void workspaceToStream(const WorkSpace& workspace, std::ostream& os);
};

class SatWriter : public Writer {
 public:
  void formulaToStream(const Formula& formula, std::ostream& os);
  void workspaceToStream(const WorkSpace& workspace, std::ostream& os);
};

class SexprWriter : public Writer {
 public:
  void formulaToStream(const Formula& formula, std::ostream& os);
  void workspaceToStream(const WorkSpace& workspace, std::ostream& os);
};

class NoncnfWriter : public Writer {
 public:
  void count(const Formula& formula, int& node);
  int visit(const Formula& formula, std::ostream& os, int& node);
  void formulaToStream(const Formula& formula, std::ostream& os);
  void workspaceToStream(const WorkSpace& workspace, std::ostream& os);
};

class IscasWriter : public Writer {
 public:
	int visit(const Formula& formula, std::ostream& os, int& node);
  void formulaToStream(const Formula& formula, std::ostream& os);
  void workspaceToStream(const WorkSpace& workspace, std::ostream& os);
};

#endif // WRITERS_H
